@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: autoclean.sh 447 2006-08-18 21:00:58Z wojtekka $
+# $Id$
 
 rm -rf \
 	aclocal.m4 \
@@ -31,6 +31,11 @@ rm -rf \
 	include/Makefile.in \
 	include/libgadu.h \
 	include/stamp* \
+	docs/Makefile.in \
+	examples/Makefile.in \
+	test/Makefile.in \
+	test/*/Makefile.in \
+	test/protocol/tests/Makefile.in \
 	pkgconfig/Makefile \
 	pkgconfig/Makefile.in \
 	pkgconfig/libgadu.pc \
