@@ -22,9 +22,24 @@
 #define LIBGADU_INTERNAL_H
 
 #include "libgadu.h"
+#include "config.h"
+
+struct gg_dcc7_relay {
+	uint32_t addr;
+	uint16_t port;
+	uint8_t family;
+};
+
+typedef struct gg_dcc7_relay gg_dcc7_relay_t;
 
-char *gg_cp_to_utf8(const char *b);
-char *gg_utf8_to_cp(const char *b);
 int gg_pubdir50_handle_reply_sess(struct gg_session *sess, struct gg_event *e, const char *packet, int length);
 
+int gg_resolve(int *fd, int *pid, const char *hostname);
+int gg_resolve_pthread(int *fd, void **resolver, const char *hostname);
+void gg_resolve_pthread_cleanup(void *resolver, int kill);
+
+#ifdef HAVE_UINT64_T
+uint64_t gg_fix64(uint64_t x);
+#endif
+
 #endif /* LIBGADU_INTERNAL_H */
