@@ -247,6 +247,9 @@ int gg_gethostbyname_real(const char *hostname, struct in_addr **result, int *co
 /**
  * \internal Rozwiązuje nazwę i zapisuje wynik do podanego desktyptora.
  *
+ * \note Użycie logowania w tej funkcji może mieć negatywny wpływ na
+ * aplikacje jednowątkowe korzystające.
+ *
  * \param fd Deskryptor
  * \param hostname Nazwa serwera
  *
@@ -258,8 +261,6 @@ static int gg_resolver_run(int fd, const char *hostname)
 	int addr_count;
 	int res = 0;
 
-	gg_debug(GG_DEBUG_MISC, "// gg_resolver_run(%d, %s)\n", fd, hostname);
-
 	if ((addr_ip[0].s_addr = inet_addr(hostname)) == INADDR_NONE) {
 		if (gg_gethostbyname_real(hostname, &addr_list, &addr_count, 1) == -1) {
 			addr_list = addr_ip;
@@ -272,8 +273,6 @@ static int gg_resolver_run(int fd, const char *hostname)
 		addr_count = 1;
 	}
 
-	gg_debug(GG_DEBUG_MISC, "// gg_resolver_run() count = %d\n", addr_count);
-
 	if (write(fd, addr_list, (addr_count + 1) * sizeof(struct in_addr)) != (addr_count + 1) * sizeof(struct in_addr))
 		res = -1;
 
