@@ -1,4 +1,4 @@
-/* $Id: sha1.c 581 2008-04-08 13:34:17Z porridge $ */
+/* $Id$ */
 
 /*
  *  (C) Copyright 2007 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -23,9 +23,10 @@
 /**
  * \file sha1.c
  *
- * \brief Funkcje wyznaczania skrĂłtu SHA1
+ * \brief Funkcje wyznaczania skrótu SHA1
  */
 
+#include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
 
